import { eq } from 'drizzle-orm';
import { db } from '..';
import { teams } from '../schema/teams';

/**
 * Get all teams for a specific competition
 * @param competitionId The ID of the competition
 * @returns Array of teams
 */
export async function getTeamsByCompetition(competitionId: string) {
	return await db.select().from(teams).where(eq(teams.competition_id, competitionId));
}

/**
 * Get a specific team by ID
 * @param teamId The ID of the team
 * @returns The team or undefined if not found
 */
export async function getTeamById(teamId: string) {
	const results = await db.select().from(teams).where(eq(teams.id, teamId));

	return results[0];
}

/**
 * Insert a new team
 * @param team The team data to insert
 * @returns The inserted team
 */
export async function insertTeam(team: { name: string; competition_id: string }) {
	return await db.insert(teams).values(team).returning();
}

/**
 * Update an existing team
 * @param teamId The ID of the team to update
 * @param data The updated team data
 * @returns The updated team
 */
export async function updateTeam(teamId: string, data: { name?: string }) {
	return await db.update(teams).set(data).where(eq(teams.id, teamId)).returning();
}

/**
 * Delete a team
 * @param teamId The ID of the team to delete
 * @returns The deleted team
 */
export async function deleteTeam(teamId: string) {
	return await db.delete(teams).where(eq(teams.id, teamId)).returning();
}
