import { db } from '../db';
import { brackets } from '../db/schema/brackets';
import { rounds } from '../db/schema/rounds';
import { matches, type Match } from '../db/schema/matches';
import { BracketType, MatchStatus, SchedulingMode } from '@/types';
import { BaseTournamentGenerator, type SeededTeam } from './types';
import type { RoundInsert } from '../db/schema/rounds';
import { v7 } from 'uuid';
import { and, desc, eq } from 'drizzle-orm';
import { fields } from '../db/schema/fields';
import type { Team } from '../db/schema/teams';

export class DoubleEliminationGenerator extends BaseTournamentGenerator {
	getMode(): SchedulingMode {
		return SchedulingMode.double;
	}

	/**
	 * Generate brackets for a double elimination tournament
	 * Double elimination requires both winners and losers brackets
	 */
	async generateBrackets(): Promise<string[]> {
		// Insert the winner's bracket
		const [winnersBracket] = await db
			.insert(brackets)
			.values({
				id: v7(),
				name: 'Winners Bracket',
				bracketType: BracketType.WINNER,
				position: 1,
				isActive: true,
				competition_id: this.competitionId,
				scheduling_mode: SchedulingMode.double
			})
			.returning();

		// Insert the losers bracket
		const [losersBracket] = await db
			.insert(brackets)
			.values({
				id: v7(),
				name: 'Losers Bracket',
				bracketType: BracketType.LOSER,
				position: 2,
				isActive: true,
				competition_id: this.competitionId,
				scheduling_mode: SchedulingMode.double
			})
			.returning();

		return [winnersBracket.id, losersBracket.id];
	}

	/**
	 * Generate rounds for a specific bracket in a double elimination tournament
	 */
	async generateRounds(bracketId: string): Promise<RoundInsert[]> {
		// Get the bracket to determine if it's winners or losers bracket
		const [bracket] = await db.select().from(brackets).where(eq(brackets.id, bracketId));

		if (!bracket) {
			throw new Error(`Bracket not found: ${bracketId}`);
		}

		const isWinnersBracket = bracket.bracketType === BracketType.WINNER;
		const createdRounds: RoundInsert[] = [];

		// Calculate the number of rounds needed
		const roundsInWinnersBracket = Math.ceil(Math.log2(this.teamsCount));

		// For winners bracket
		if (isWinnersBracket) {
			// Create all rounds for the winners bracket
			for (let i = 0; i < roundsInWinnersBracket; i++) {
				const roundNumber = i + 1;
				let roundName = '';

				// Name rounds appropriately
				if (i === roundsInWinnersBracket - 1) {
					roundName = 'Winners Finals';
				} else if (i === roundsInWinnersBracket - 2) {
					roundName = 'Winners Semi-Finals';
				} else {
					roundName = `Winners Round ${roundNumber}`;
				}

				// Calculate matches in this round
				const matchesInRound = Math.floor(this.teamsCount / Math.pow(2, i));

				const roundData: RoundInsert = {
					name: roundName,
					round_number: roundNumber,
					nb_matches: matchesInRound,
					bracket_id: bracketId
				};

				const [insertedRound] = await db.insert(rounds).values(roundData).returning();
				createdRounds.push(insertedRound);
			}

			// Add a Grand Finals round for the winners bracket
			const grandFinalsRound: RoundInsert = {
				name: 'Grand Finals',
				round_number: roundsInWinnersBracket + 1,
				nb_matches: 1,
				bracket_id: bracketId
			};

			const [insertedGrandFinals] = await db.insert(rounds).values(grandFinalsRound).returning();
			createdRounds.push(insertedGrandFinals);

			// Add a potential reset bracket for the Grand Finals (if loser of winners bracket wins)
			const resetRound: RoundInsert = {
				name: 'Grand Finals Reset',
				round_number: roundsInWinnersBracket + 2,
				nb_matches: 1,
				bracket_id: bracketId
			};

			const [insertedReset] = await db.insert(rounds).values(resetRound).returning();
			createdRounds.push(insertedReset);
		}
		// For losers bracket
		else {
			// Losers bracket has 2 * log2(n) - 1 rounds
			const totalLosersRounds = 2 * roundsInWinnersBracket - 1;

			for (let i = 0; i < totalLosersRounds; i++) {
				const roundNumber = i + 1;
				let roundName = '';

				// Name rounds appropriately
				if (i === totalLosersRounds - 1) {
					roundName = 'Losers Finals';
				} else if (i === totalLosersRounds - 2) {
					roundName = 'Losers Semi-Finals';
				} else {
					roundName = `Losers Round ${roundNumber}`;
				}

				// Calculate matches in this round - more complex for losers bracket
				// In double elimination, losers bracket structure depends on the round
				let matchesInRound: number;

				if (i % 2 === 0) {
					// Rounds where losers from winners bracket enter
					matchesInRound = Math.floor(this.teamsCount / Math.pow(2, Math.floor(i / 2) + 1));
				} else {
					// Rounds where losers bracket teams play each other
					matchesInRound = Math.floor(this.teamsCount / Math.pow(2, Math.floor(i / 2) + 2));
				}

				// Ensure at least 1 match
				matchesInRound = Math.max(1, matchesInRound);

				const roundData: RoundInsert = {
					name: roundName,
					round_number: roundNumber,
					nb_matches: matchesInRound,
					bracket_id: bracketId
				};

				const [insertedRound] = await db.insert(rounds).values(roundData).returning();
				createdRounds.push(insertedRound);
			}
		}

		return createdRounds;
	}

	/**
	 * Generate matches for the first round of a bracket
	 */
	async generateMatches(roundId: string, teams: Team[]): Promise<Match[]> {
		// Get the round
		const [round] = await db.select().from(rounds).where(eq(rounds.id, roundId));

		if (!round) {
			throw new Error(`Round not found: ${roundId}`);
		}

		if (!round.bracket_id) {
			throw new Error(`Round ${roundId} has no bracket`);
		}

		// Get the bracket
		const [bracket] = await db.select().from(brackets).where(eq(brackets.id, round.bracket_id));

		if (!bracket) {
			throw new Error(`Bracket not found for round: ${roundId}`);
		}

		// Get a default field
		const [defaultField] = await db
			.select()
			.from(fields)
			.where(eq(fields.competition_id, this.competitionId))
			.limit(1);

		if (!defaultField) {
			throw new Error('No field available for matches');
		}

		// For winners bracket first round, seed teams appropriately
		if (bracket.bracketType === BracketType.WINNER && round.round_number === 1) {
			// Create seeded teams
			const seededTeams: SeededTeam[] = teams.map((team, index) => ({
				teamId: team.id,
				seed: index + 1
			}));

			// Generate optimal pairings for single elimination (same as first round of double elimination)
			const pairings = this.createDoubleEliminationPairings(seededTeams);

			// Create matches
			const matchesToCreate: Match[] = pairings.map((pairing, index) => {
				const now = new Date();
				const endTime = new Date(now);
				endTime.setHours(endTime.getHours() + 1);

				return {
					id: v7(),
					match_number: index + 1,
					field_id: defaultField.id,
					team1_id: pairing.team1Id,
					team2_id: pairing.team2Id || '00000000-0000-0000-0000-000000000000', // Default "bye" UUID if needed
					winner_id: null,
					start_date: now,
					end_date: endTime,
					score1: 0,
					score2: 0,
					status: MatchStatus.PENDING,
					position: index + 1,
					table: 0, // Not relevant for elimination tournaments
					round_id: roundId,
					bracket_id: bracket.id,
					created_at: now,
					updated_at: null,
					deleted_at: null
				};
			});

			// Insert matches and return them
			const createdMatches = await db.insert(matches).values(matchesToCreate).returning();
			return createdMatches;
		}

		// For losers bracket first round, this is generated based on results from winners bracket
		// This would be handled separately by a specialized method

		throw new Error('For rounds after the first round or losers bracket, use generateNextRound');
	}

	/**
	 * Generate the next round based on previous round results
	 * This is complex for double elimination as it involves tracking both winners and losers
	 */
	async generateNextRound(previousRoundId: string, bracketId: string): Promise<RoundInsert | null> {
		// Get the previous round
		const [previousRound] = await db.select().from(rounds).where(eq(rounds.id, previousRoundId));

		if (!previousRound) {
			throw new Error(`Previous round not found: ${previousRoundId}`);
		}

		// Get the bracket
		const [bracket] = await db.select().from(brackets).where(eq(brackets.id, bracketId));

		if (!bracket) {
			throw new Error(`Bracket not found: ${bracketId}`);
		}

		// Get all rounds in the current bracket
		const allRounds = await db
			.select()
			.from(rounds)
			.where(eq(rounds.bracket_id, bracketId))
			.orderBy(rounds.round_number);

		// Find the next round
		const nextRound = allRounds.find((r) => r.round_number === previousRound.round_number + 1);

		if (!nextRound) {
			// This could happen if we're at the final round of a bracket
			return null;
		}

		// Get previous round's matches
		const previousMatches = await db
			.select()
			.from(matches)
			.where(eq(matches.round_id, previousRoundId))
			.orderBy(matches.position);

		// Check if all matches have results
		const allMatchesComplete = previousMatches.every((match) => match.winner_id);
		if (!allMatchesComplete) {
			throw new Error(
				'Cannot generate next round until all matches in the current round have winners'
			);
		}

		// Default field
		const [defaultField] = await db
			.select()
			.from(fields)
			.where(eq(fields.competition_id, this.competitionId))
			.limit(1);

		if (!defaultField) {
			throw new Error('No field available for matches');
		}

		// Handle based on bracket type
		if (bracket.bracketType === BracketType.WINNER) {
			// For winners bracket, winners advance to next round
			const matchesToCreate: Match[] = [];

			// Special case for Grand Finals
			if (nextRound.name === 'Grand Finals') {
				// Get winners bracket finalist
				const winnersBracketFinalist = previousMatches[0].winner_id;
				if (!winnersBracketFinalist) {
					throw new Error('No winners bracket finalist found');
				}

				// Get losers bracket finalist
				const losersBrackets = await db
					.select()
					.from(brackets)
					.where(
						and(
							eq(brackets.competition_id, this.competitionId),
							eq(brackets.bracketType, BracketType.LOSER)
						)
					);

				if (losersBrackets.length === 0) {
					throw new Error('No losers bracket found for grand finals');
				}

				const losersFinals = await db
					.select()
					.from(rounds)
					.where(eq(rounds.bracket_id, losersBrackets[0].id))
					.orderBy(desc(rounds.round_number))
					.limit(1);

				if (losersFinals.length === 0) {
					throw new Error('No losers finals found');
				}

				const losersFinalsMatches = await db
					.select()
					.from(matches)
					.where(eq(matches.round_id, losersFinals[0].id));

				if (losersFinalsMatches.length === 0 || !losersFinalsMatches[0].winner_id) {
					throw new Error('Losers finalist not determined yet');
				}

				const losersBracketFinalist = losersFinalsMatches[0].winner_id;

				// Create grand finals match
				const now = new Date();
				const endTime = new Date(now);
				endTime.setHours(endTime.getHours() + 1);

				matchesToCreate.push({
					id: v7(),
					match_number: 1,
					team1_id: winnersBracketFinalist,
					team2_id: losersBracketFinalist,
					start_date: now,
					end_date: endTime,
					position: 1,
					table: 0,
					round_id: nextRound.id,
					bracket_id: bracketId,
					field_id: defaultField.id,
					score1: 0,
					score2: 0,
					status: MatchStatus.PENDING,
					winner_id: null,
					created_at: now,
					updated_at: null,
					deleted_at: null
				});
			}
			// For Reset bracket (if loser's bracket winner wins grand finals)
			else if (nextRound.name === 'Grand Finals Reset') {
				// This would be triggered after grand finals if the loser's bracket winner wins
				// Implementation depends on your reset bracket logic
			}
			// Normal winners bracket progression
			else {
				// Create next round matches by pairing winners
				for (let i = 0; i < nextRound.nb_matches; i++) {
					const team1Id = previousMatches[i * 2].winner_id;
					const team2Id = previousMatches[i * 2 + 1].winner_id;

					if (!team1Id || !team2Id) {
						throw new Error('Missing team IDs');
					}

					const now = new Date();
					const endTime = new Date(now);
					endTime.setHours(endTime.getHours() + 1);

					matchesToCreate.push({
						id: v7(),
						match_number: i + 1,
						team1_id: team1Id,
						team2_id: team2Id,
						start_date: now,
						end_date: endTime,
						position: i + 1,
						table: 0,
						round_id: nextRound.id,
						bracket_id: bracketId,
						field_id: defaultField.id,
						score1: 0,
						score2: 0,
						status: MatchStatus.PENDING,
						winner_id: null,
						created_at: now,
						updated_at: null,
						deleted_at: null
					});
				}

				// Also send losers to the losers bracket
				await this.sendLosersToLosersBracket(previousMatches, previousRound.round_number);
			}

			if (matchesToCreate.length > 0) {
				await db.insert(matches).values(matchesToCreate);
			}
		} else if (bracket.bracketType === BracketType.LOSER) {
			// Losers bracket logic is more complex and depends on the round
			// In odd-numbered rounds, losers from winners bracket join
			// In even-numbered rounds, losers bracket teams face each other

			const matchesToCreate: Match[] = [];

			// Logic varies based on the round number
			if (previousRound.round_number % 2 === 1) {
				// In odd rounds, winners advance to next round
				for (let i = 0; i < nextRound.nb_matches; i++) {
					// If there are enough matches in the previous round
					if (i * 2 + 1 < previousMatches.length) {
						const team1Id = previousMatches[i * 2].winner_id;
						const team2Id = previousMatches[i * 2 + 1].winner_id;

						if (!team1Id || !team2Id) {
							throw new Error('Missing team IDs');
						}

						const now = new Date();
						const endTime = new Date(now);
						endTime.setHours(endTime.getHours() + 1);

						matchesToCreate.push({
							id: v7(),
							match_number: i + 1,
							team1_id: team1Id,
							team2_id: team2Id,
							start_date: now,
							end_date: endTime,
							position: i + 1,
							table: 0,
							round_id: nextRound.id,
							bracket_id: bracketId,
							field_id: defaultField.id,
							score1: 0,
							score2: 0,
							status: MatchStatus.PENDING,
							winner_id: null,
							created_at: now,
							updated_at: null,
							deleted_at: null
						});
					}
				}
			} else {
				// In even rounds, losers from winners bracket join
				// This requires looking up the corresponding winners bracket matches
				// Implementation would depend on your exact double elimination structure
				// This is a complex part that varies by tournament design
			}

			if (matchesToCreate.length > 0) {
				await db.insert(matches).values(matchesToCreate);
			}
		}

		return nextRound;
	}

	/**
	 * Send losers from winners bracket to losers bracket
	 * This is a key part of double elimination
	 */
	private async sendLosersToLosersBracket(
		winnersBracketMatches: any[],
		winnersRoundNumber: number
	): Promise<void> {
		// Get the losers bracket
		const [losersBracket] = await db
			.select()
			.from(brackets)
			.where(
				and(
					eq(brackets.competition_id, this.competitionId),
					eq(brackets.bracketType, BracketType.LOSER)
				)
			);

		if (!losersBracket) {
			throw new Error('No losers bracket found');
		}

		// Find the appropriate losers bracket round
		// In standard double elimination, losers from winners round N go to losers round 2N-1
		const targetLosersRound = winnersRoundNumber * 2 - 1;

		const [losersRound] = await db
			.select()
			.from(rounds)
			.where(
				and(eq(rounds.bracket_id, losersBracket.id), eq(rounds.round_number, targetLosersRound))
			);

		if (!losersRound) {
			throw new Error(`No losers round found for winners round ${winnersRoundNumber}`);
		}

		// Get a default field
		const [defaultField] = await db
			.select()
			.from(fields)
			.where(eq(fields.competition_id, this.competitionId))
			.limit(1);

		if (!defaultField) {
			throw new Error('No field available for matches');
		}

		// Create matches in the losers bracket with the losers from winners bracket
		const matchesToCreate: Match[] = [];
		let matchCount = 0;

		// For each match in the winners bracket, get the loser
		for (let i = 0; i < winnersBracketMatches.length; i++) {
			const match = winnersBracketMatches[i];
			if (!match.winner_id) continue;

			// Determine the loser
			const loserId = match.winner_id === match.team1_id ? match.team2_id : match.team1_id;

			// In first losers round, each loser gets their own match (often with a bye)
			if (targetLosersRound === 1) {
				const now = new Date();
				const endTime = new Date(now);
				endTime.setHours(endTime.getHours() + 1);

				matchesToCreate.push({
					id: v7(),
					match_number: matchCount + 1,
					team1_id: loserId,
					team2_id: '00000000-0000-0000-0000-000000000000', // Initial bye or determined by bracket structure
					start_date: now,
					end_date: endTime,
					position: matchCount + 1,
					table: 0,
					round_id: losersRound.id,
					bracket_id: losersBracket.id,
					field_id: defaultField.id,
					score1: 0,
					score2: 0,
					status: MatchStatus.PENDING,
					winner_id: null,
					created_at: now,
					updated_at: null,
					deleted_at: null
				});

				matchCount++;
			}
			// In later rounds, losers typically play against survivors from previous losers rounds
			// This would be handled in the main generateNextRound method for the losers bracket
		}

		if (matchesToCreate.length > 0) {
			await db.insert(matches).values(matchesToCreate);
		}
	}

	/**
	 * Create optimal pairings for the first round of double elimination
	 * Uses standard tournament seeding to ensure balanced bracket
	 */
	private createDoubleEliminationPairings(
		seededTeams: SeededTeam[]
	): { team1Id: string; team2Id: string | null }[] {
		// For first round, use same seeding as single elimination
		const totalTeamsInFullBracket = Math.pow(2, this.calculateRoundsNeeded());
		const pairings: { team1Id: string; team2Id: string | null }[] = [];

		// Sort teams by seed
		const sortedTeams = [...seededTeams].sort((a, b) => a.seed - b.seed);

		// If we need to give byes, calculate them
		const teamsWithByes = sortedTeams.slice(0, 2 * sortedTeams.length - totalTeamsInFullBracket);
		const teamsWithoutByes = sortedTeams.slice(teamsWithByes.length);

		// Add matches for teams with direct advancement (byes)
		for (const team of teamsWithByes) {
			pairings.push({
				team1Id: team.teamId,
				team2Id: null // Bye
			});
		}

		// Create standard bracket pairings for remaining teams
		for (let i = 0; i < teamsWithoutByes.length; i += 2) {
			pairings.push({
				team1Id: teamsWithoutByes[i].teamId,
				team2Id: teamsWithoutByes[i + 1].teamId
			});
		}

		return pairings;
	}
}
