import { db } from '../db';
import { brackets } from '../db/schema/brackets';
import { rounds } from '../db/schema/rounds';
import { matches, type Match } from '../db/schema/matches';
import { BracketType, MatchStatus, SchedulingMode } from '@/types';
import { BaseTournamentGenerator, type SeededTeam } from './types';
import type { RoundInsert } from '../db/schema/rounds';
import { fields } from '../db/schema/fields';
import { v7 } from 'uuid';
import { eq } from 'drizzle-orm';
import type { Team } from '../db/schema/teams';

export class SingleEliminationGenerator extends BaseTournamentGenerator {
	getMode(): SchedulingMode {
		return SchedulingMode.single;
	}

	/**
	 * Generate a single bracket for the single elimination tournament
	 */
	async generateBrackets(): Promise<string[]> {
		// Insert the winner's bracket
		const [bracket] = await db
			.insert(brackets)
			.values({
				name: 'Winner Bracket',
				bracketType: BracketType.WINNER,
				position: 1,
				isActive: true,
				competition_id: this.competitionId,
				scheduling_mode: SchedulingMode.single
			})
			.returning();

		return [bracket.id];
	}

	/**
	 * Generate all rounds needed for a single elimination tournament
	 */
	async generateRounds(bracketId: string): Promise<RoundInsert[]> {
		const totalRounds = this.calculateRoundsNeeded();
		const createdRounds: RoundInsert[] = [];

		for (let i = 0; i < totalRounds; i++) {
			const roundNumber = i + 1;
			let roundName = '';

			// Name the rounds appropriately
			if (i === totalRounds - 1) {
				roundName = 'Final';
			} else if (i === totalRounds - 2) {
				roundName = 'Semi-Finals';
			} else if (i === totalRounds - 3) {
				roundName = 'Quarter-Finals';
			} else {
				roundName = `Round ${roundNumber}`;
			}

			// Calculate number of matches in this round
			const matchesInRound = Math.floor(this.teamsCount / Math.pow(2, i));

			const roundData: RoundInsert = {
				name: roundName,
				round_number: roundNumber,
				nb_matches: matchesInRound,
				bracket_id: bracketId
			};

			// Insert the round
			const [insertedRound] = await db.insert(rounds).values(roundData).returning();
			createdRounds.push(insertedRound);
		}

		return createdRounds;
	}

	/**
	 * Generate matches for the first round of a single elimination tournament
	 * For subsequent rounds, use generateNextRound
	 */
	async generateMatches(roundId: string, teams: Team[]): Promise<Match[]> {
		// Get the round
		const [round] = await db.select().from(rounds).where(eq(rounds.id, roundId));

		if (!round) {
			throw new Error(`Round not found: ${roundId}`);
		}

		if (!round.bracket_id) {
			throw new Error(`Round has no bracket: ${roundId}`);
		}

		// Get the bracket
		const [bracket] = await db.select().from(brackets).where(eq(brackets.id, round.bracket_id));

		if (!bracket) {
			throw new Error(`Bracket not found for round: ${roundId}`);
		}

		// Seed the teams (this is a simple seeding, could be enhanced)
		const seededTeams: SeededTeam[] = teams.map((team, index) => ({
			teamId: team.id,
			seed: index + 1
		}));

		// Distribute teams according to standard tournament seeding
		const pairings = this.createSingleEliminationPairings(seededTeams, round.round_number);

		// Default field (could be enhanced to assign different fields)
		const [defaultField] = await db
			.select()
			.from(fields)
			.where(eq(fields.competition_id, this.competitionId))
			.limit(1);

		if (!defaultField) {
			throw new Error('No field available for matches');
		}

		// Create match objects
		const matchesToCreate: Match[] = pairings.map((pairing, index) => {
			const now = new Date();
			const endTime = new Date(now);
			endTime.setHours(endTime.getHours() + 1); // Default 1-hour match

			return {
				id: v7(),
				match_number: index + 1,
				team1_id: pairing.team1Id,
				team2_id: pairing.team2Id || '00000000-0000-0000-0000-000000000000', // Default "bye" UUID if needed
				start_date: now,
				end_date: endTime,
				position: index + 1,
				table: 0, // Not relevant for elimination tournaments
				round_id: roundId,
				bracket_id: bracket.id,
				field_id: defaultField.id,
				score1: 0,
				score2: 0,
				status: MatchStatus.PENDING,
				winner_id: null,
				created_at: now,
				updated_at: null,
				deleted_at: null
			};
		});

		// Insert matches and return them
		const createdMatches = await db.insert(matches).values(matchesToCreate).returning();
		return createdMatches;
	}

	/**
	 * Generate the next round based on the results of a previous round
	 */
	async generateNextRound(previousRoundId: string, bracketId: string): Promise<RoundInsert | null> {
		// Get the previous round
		const [previousRound] = await db.select().from(rounds).where(eq(rounds.id, previousRoundId));

		if (!previousRound) {
			throw new Error(`Previous round not found: ${previousRoundId}`);
		}

		// Check if this was the final round
		if (previousRound.nb_matches === 1) {
			return null; // Tournament is complete
		}

		// Get the previous round's matches
		const previousMatches = await db
			.select()
			.from(matches)
			.where(eq(matches.round_id, previousRoundId))
			.orderBy(matches.position);

		// Check if all matches have winners
		const allMatchesComplete = previousMatches.every((match) => match.winner_id);

		if (!allMatchesComplete) {
			throw new Error(
				'Cannot generate next round until all matches in the current round have winners'
			);
		}

		// Create the next round
		const nextRoundNumber = previousRound.round_number + 1;
		const nextRoundMatchCount = Math.floor(previousRound.nb_matches / 2);

		let roundName = '';
		if (nextRoundMatchCount === 1) {
			roundName = 'Final';
		} else if (nextRoundMatchCount === 2) {
			roundName = 'Semi-Finals';
		} else if (nextRoundMatchCount === 4) {
			roundName = 'Quarter-Finals';
		} else {
			roundName = `Round ${nextRoundNumber}`;
		}

		const nextRoundData: RoundInsert = {
			name: roundName,
			round_number: nextRoundNumber,
			nb_matches: nextRoundMatchCount,
			bracket_id: bracketId
		};

		// Insert the next round
		const [nextRound] = await db.insert(rounds).values(nextRoundData).returning();

		// Create matches for the next round
		const nextRoundMatches: Match[] = [];

		// Create pairings for the next round based on winners
		for (let i = 0; i < nextRoundMatchCount; i++) {
			const team1Id = previousMatches[i * 2].winner_id;
			const team2Id = previousMatches[i * 2 + 1].winner_id;

			if (!team1Id || !team2Id) {
				throw new Error('Missing winner ID for matches');
			}

			const now = new Date();
			const endTime = new Date(now);
			endTime.setHours(endTime.getHours() + 1);

			// Get the same field from previous match or default
			const field_id = previousMatches[i].field_id;

			nextRoundMatches.push({
				id: v7(),
				match_number: i + 1,
				team1_id: team1Id,
				team2_id: team2Id,
				start_date: now,
				end_date: endTime,
				position: i + 1,
				table: 0,
				round_id: nextRound.id,
				bracket_id: bracketId,
				field_id,
				score1: 0,
				score2: 0,
				status: MatchStatus.PENDING,
				winner_id: null,
				created_at: now,
				updated_at: null,
				deleted_at: null
			});
		}

		// Insert the matches for the next round
		await db.insert(matches).values(nextRoundMatches);

		return nextRound;
	}

	/**
	 * Creates optimal pairings for a single elimination tournament
	 * Uses the standard tournament bracket seeding algorithm
	 */
	private createSingleEliminationPairings(
		seededTeams: SeededTeam[],
		roundNumber: number
	): { team1Id: string; team2Id: string | null }[] {
		// Handle first round with proper seeding
		if (roundNumber === 1) {
			const totalTeamsInFullBracket = seededTeams.length;
			const pairings: { team1Id: string; team2Id: string | null }[] = [];
			const nextPowerOfTwo = 1 << Math.ceil(Math.log2(totalTeamsInFullBracket));
			// Sort teams by seed
			const sortedTeams = [...seededTeams].sort((a, b) => a.seed - b.seed);

			if (sortedTeams.length % 2 !== 0) {
				sortedTeams.push({
					teamId: '00000000-0000-0000-0000-000000000000',
					seed: sortedTeams.length + 1
				});
			}

			for (let i = 0; i < nextPowerOfTwo; i += 2) {
				pairings.push({
					team1Id: sortedTeams[i].teamId,
					team2Id: sortedTeams[i + 1].teamId
				});
			}

			return pairings;
		}

		// For other rounds, this should not be called directly
		// Use generateNextRound instead
		throw new Error('Use generateNextRound for rounds after the first round');
	}
}
