import { db } from '../db';
import { brackets } from '../db/schema/brackets';
import { rounds } from '../db/schema/rounds';
import { matches, type Match } from '../db/schema/matches';
import { BracketType, MatchStatus, SchedulingMode } from '@/types';
import { BaseTournamentGenerator } from './types';
import type { RoundInsert } from '../db/schema/rounds';
import { fields } from '../db/schema/fields';
import { v7 } from 'uuid';
import { and, eq } from 'drizzle-orm';
import { teams, type Team } from '../db/schema/teams';

export class SwissGenerator extends BaseTournamentGenerator {
	getMode(): SchedulingMode {
		return SchedulingMode.swiss;
	}

	/**
	 * Generate a single main bracket for Swiss tournament
	 */
	async generateBrackets(): Promise<string[]> {
		const [bracket] = await db
			.insert(brackets)
			.values({
				name: 'Swiss Bracket',
				bracketType: BracketType.MAIN,
				position: 1,
				isActive: true,
				competition_id: this.competitionId,
				scheduling_mode: SchedulingMode.swiss
			})
			.returning();

		return [bracket.id];
	}

	/**
	 * Generate rounds for a Swiss tournament
	 * In Swiss, the number of rounds is typically log2(n), where n is the number of teams
	 * This ensures that a single undefeated team will emerge
	 */
	async generateRounds(bracketId: string): Promise<RoundInsert[]> {
		// Recommended number of rounds is log2(n) rounded up
		const recommendedRounds = Math.ceil(Math.log2(this.teamsCount));
		const totalRounds = Math.min(recommendedRounds, this.teamsCount - 1);
		const createdRounds: RoundInsert[] = [];

		for (let i = 0; i < totalRounds; i++) {
			const roundNumber = i + 1;
			const roundName = `Round ${roundNumber}`;

			// Each round has floor(n/2) matches
			const matchesInRound = Math.floor(this.teamsCount / 2);

			const roundData: RoundInsert = {
				name: roundName,
				round_number: roundNumber,
				nb_matches: matchesInRound,
				bracket_id: bracketId
			};

			const [insertedRound] = await db.insert(rounds).values(roundData).returning();
			createdRounds.push(insertedRound);
		}

		return createdRounds;
	}

	/**
	 * Generate matches for a Swiss tournament
	 * First round is random or seeded pairings
	 * Subsequent rounds are handled by generateNextRound
	 */
	async generateMatches(roundId: string, teams: Team[]): Promise<Match[]> {
		// Get the round
		const [round] = await db.select().from(rounds).where(eq(rounds.id, roundId));

		if (!round) {
			throw new Error(`Round not found: ${roundId}`);
		}

		if (!round.bracket_id) {
			throw new Error(`Round ${roundId} has no bracket`);
		}

		// Get the bracket
		const [bracket] = await db.select().from(brackets).where(eq(brackets.id, round.bracket_id));

		if (!bracket) {
			throw new Error(`Bracket not found for round: ${roundId}`);
		}

		// Default field (could be enhanced to assign different fields)
		const [defaultField] = await db
			.select()
			.from(fields)
			.where(eq(fields.competition_id, this.competitionId))
			.limit(1);

		if (!defaultField) {
			throw new Error('No field available for matches');
		}

		// For first round, generate random pairings
		// In a real implementation, this could be seeded pairings
		const pairings = this.generateFirstRoundPairings(teams);

		// Create match objects
		const matchesToCreate: Match[] = pairings.map((pairing, index) => {
			const now = new Date();
			const endTime = new Date(now);
			endTime.setHours(endTime.getHours() + 1); // Default 1-hour match

			return {
				id: v7(),
				match_number: index + 1,
				team1_id: pairing.team1.id,
				team2_id: pairing.team2?.id || '00000000-0000-0000-0000-000000000000', // Default "bye" UUID if needed
				start_date: now,
				end_date: endTime,
				position: index + 1,
				table: index + 1, // Table assignment
				round_id: roundId,
				bracket_id: bracket.id,
				field_id: defaultField.id,
				score1: 0,
				score2: 0,
				status: MatchStatus.PENDING,
				winner_id: null,
				created_at: now,
				updated_at: null,
				deleted_at: null
			};
		});

		// Insert matches and return them
		const createdMatches = await db.insert(matches).values(matchesToCreate).returning();
		return createdMatches;
	}

	/**
	 * Generate the next round based on current standings
	 * This is the core of the Swiss system - pair teams with similar records
	 */
	async generateNextRound(previousRoundId: string, bracketId: string): Promise<RoundInsert | null> {
		// Get the previous round
		const [previousRound] = await db.select().from(rounds).where(eq(rounds.id, previousRoundId));

		if (!previousRound) {
			throw new Error(`Previous round not found: ${previousRoundId}`);
		}

		// Get all rounds in the bracket
		const allRounds = await db
			.select()
			.from(rounds)
			.where(eq(rounds.bracket_id, bracketId))
			.orderBy(rounds.round_number);

		// Check if this was the final round
		if (previousRound.round_number === allRounds.length) {
			return null; // Tournament is complete
		}

		// Find the next round
		const nextRound = allRounds.find((r) => r.round_number === previousRound.round_number + 1);

		if (!nextRound) {
			throw new Error(`Next round not found for round ${previousRound.round_number}`);
		}

		// Calculate standings to generate pairings
		const standings = await this.calculateStandings(bracketId);

		// Generate pairings based on standings
		const pairings = this.generateSwissPairings(standings);

		// Default field
		const [defaultField] = await db
			.select()
			.from(fields)
			.where(eq(fields.competition_id, this.competitionId))
			.limit(1);

		if (!defaultField) {
			throw new Error('No field available for matches');
		}

		// Create matches for the next round
		const matchesToCreate: Match[] = pairings.map((pairing, index) => {
			const now = new Date();
			const endTime = new Date(now);
			endTime.setHours(endTime.getHours() + 1);

			return {
				id: v7(),
				match_number: index + 1,
				team1_id: pairing.team1Id,
				team2_id: pairing.team2Id || '00000000-0000-0000-0000-000000000000',
				start_date: now,
				end_date: endTime,
				position: index + 1,
				table: index + 1,
				round_id: nextRound.id,
				bracket_id: bracketId,
				field_id: defaultField.id,
				score1: 0,
				score2: 0,
				status: MatchStatus.PENDING,
				winner_id: null,
				created_at: now,
				updated_at: null,
				deleted_at: null
			};
		});

		// Insert the matches for the next round
		await db.insert(matches).values(matchesToCreate);

		return nextRound;
	}

	/**
	 * Generate pairings for the first round
	 * In Swiss, the first round can be random or seeded
	 */
	private generateFirstRoundPairings(teams: Team[]): { team1: Team; team2: Team | null }[] {
		const pairings: { team1: Team; team2: Team | null }[] = [];

		// Handle odd number of teams
		if (teams.length % 2 !== 0) {
			// Give the bye to a random team
			const byeTeamIndex = Math.floor(Math.random() * teams.length);
			const byeTeam = teams.splice(byeTeamIndex, 1)[0];
			pairings.push({ team1: byeTeam, team2: null });
		}

		// Shuffle the remaining teams for random pairings
		// In a real system, this could be seeded based on team rankings
		this.shuffleArray(teams);

		// Create pairings
		for (let i = 0; i < teams.length; i += 2) {
			pairings.push({
				team1: teams[i],
				team2: teams[i + 1]
			});
		}

		return pairings;
	}

	/**
	 * Calculate current standings for all teams
	 * This is a key component for Swiss pairings
	 */
	private async calculateStandings(bracketId: string): Promise<TeamStanding[]> {
		// Get all completed matches in the bracket
		const completedMatches = await db
			.select()
			.from(matches)
			.where(and(eq(matches.bracket_id, bracketId), eq(matches.status, MatchStatus.FINISHED)));

		// Get all teams in the competition
		const teamList = await db
			.select()
			.from(teams)
			.where(eq(teams.competition_id, this.competitionId));

		// Calculate standings
		const standings: Record<string, TeamStanding> = {};

		// Initialize standings for all teams
		teamList.forEach((team) => {
			standings[team.id] = {
				teamId: team.id,
				wins: 0,
				losses: 0,
				draws: 0,
				points: 0,
				opponentWinPercentage: 0, // For tiebreakers
				previousOpponents: []
			};
		});

		// Update standings based on match results
		completedMatches.forEach((match) => {
			const team1 = standings[match.team1_id];
			const team2 = standings[match.team2_id];

			// Record that these teams played each other
			if (team1 && team2) {
				team1.previousOpponents.push(match.team2_id);
				team2.previousOpponents.push(match.team1_id);
			}

			// If there's a winner, update win/loss records
			if (match.winner_id) {
				if (match.winner_id === match.team1_id) {
					if (team1) {
						team1.wins++;
						team1.points += 3; // Typical scoring: 3 points for a win
					}
					if (team2) {
						team2.losses++;
					}
				} else {
					if (team2) {
						team2.wins++;
						team2.points += 3;
					}
					if (team1) {
						team1.losses++;
					}
				}
			} else if (match.score1 === match.score2) {
				// Draw
				if (team1) {
					team1.draws++;
					team1.points += 1; // Typical scoring: 1 point for a draw
				}
				if (team2) {
					team2.draws++;
					team2.points += 1;
				}
			}
		});

		// Calculate opponent win percentage (Buchholz tiebreaker)
		Object.values(standings).forEach((standing) => {
			if (standing.previousOpponents.length > 0) {
				const opponentWins = standing.previousOpponents.reduce((sum, opponentId) => {
					return sum + (standings[opponentId]?.wins || 0);
				}, 0);
				const opponentMatches = standing.previousOpponents.reduce((sum, opponentId) => {
					const opponent = standings[opponentId];
					return sum + (opponent ? opponent.wins + opponent.losses + opponent.draws : 0);
				}, 0);

				standing.opponentWinPercentage = opponentMatches > 0 ? opponentWins / opponentMatches : 0;
			}
		});

		return Object.values(standings);
	}

	/**
	 * Generate Swiss pairings based on current standings
	 * Pairs teams with similar records while avoiding rematches
	 */
	private generateSwissPairings(
		standings: TeamStanding[]
	): { team1Id: string; team2Id: string | null }[] {
		// Sort standings by points (descending), then by tiebreaker
		const sortedStandings = [...standings].sort((a, b) => {
			if (a.points !== b.points) {
				return b.points - a.points; // Sort by points (descending)
			}
			return b.opponentWinPercentage - a.opponentWinPercentage; // Then by opponent win %
		});

		const pairings: { team1Id: string; team2Id: string | null }[] = [];
		const unpairedTeams = [...sortedStandings];

		// Handle odd number of teams
		if (unpairedTeams.length % 2 !== 0) {
			// Find the lowest-ranked team without a bye yet
			// In a real implementation, you would track byes across rounds
			const byeTeamIndex = unpairedTeams.length - 1;
			const byeTeam = unpairedTeams.splice(byeTeamIndex, 1)[0];
			pairings.push({ team1Id: byeTeam.teamId, team2Id: null });
		}

		// Group teams by points
		const teamsByPoints: Record<number, TeamStanding[]> = {};
		unpairedTeams.forEach((team) => {
			if (!teamsByPoints[team.points]) {
				teamsByPoints[team.points] = [];
			}
			teamsByPoints[team.points].push(team);
		});

		// Sort point groups in descending order
		const pointGroups = Object.keys(teamsByPoints)
			.map(Number)
			.sort((a, b) => b - a);

		// Try to pair within same point group first
		for (const points of pointGroups) {
			const teamsInGroup = teamsByPoints[points];

			// For each team in this group
			while (teamsInGroup.length > 0) {
				const team1 = teamsInGroup.shift()!;
				let paired = false;

				// Try to find a valid opponent in the same group
				for (let i = 0; i < teamsInGroup.length; i++) {
					const team2 = teamsInGroup[i];

					// Check if these teams have already played each other
					if (!team1.previousOpponents.includes(team2.teamId)) {
						// Valid pairing found
						teamsInGroup.splice(i, 1);
						pairings.push({ team1Id: team1.teamId, team2Id: team2.teamId });
						paired = true;
						break;
					}
				}

				// If no valid pairing found in the same group
				if (!paired) {
					// Put this team back and we'll handle cross-group pairings later
					teamsInGroup.unshift(team1);
					break;
				}
			}
		}

		// Handle remaining unpaired teams (cross-group pairings)
		const remainingTeams = pointGroups.flatMap((points) => teamsByPoints[points]);

		while (remainingTeams.length > 0) {
			const team1 = remainingTeams.shift()!;

			// Find the best opponent that hasn't already played against team1
			let bestOpponentIndex = -1;

			for (let i = 0; i < remainingTeams.length; i++) {
				if (!team1.previousOpponents.includes(remainingTeams[i].teamId)) {
					bestOpponentIndex = i;
					break;
				}
			}

			// If no valid opponent is found, this shouldn't happen with proper implementation
			// In a real system, you might relax constraints or create a "forced" pairing
			if (bestOpponentIndex === -1) {
				bestOpponentIndex = 0; // Force a rematch as last resort
			}

			const team2 = remainingTeams.splice(bestOpponentIndex, 1)[0];
			pairings.push({ team1Id: team1.teamId, team2Id: team2.teamId });
		}

		return pairings;
	}

	/**
	 * Utility function to shuffle an array (Fisher-Yates algorithm)
	 */
	private shuffleArray<T>(array: T[]): T[] {
		for (let i = array.length - 1; i > 0; i--) {
			const j = Math.floor(Math.random() * (i + 1));
			[array[i], array[j]] = [array[j], array[i]];
		}
		return array;
	}
}

/**
 * Interface representing a team's standing in a Swiss tournament
 */
interface TeamStanding {
	teamId: string;
	wins: number;
	losses: number;
	draws: number;
	points: number;
	opponentWinPercentage: number;
	previousOpponents: string[];
}
