import { BracketType, SchedulingMode } from '@/types';
import type { RoundInsert } from '../db/schema/rounds';
import type { Match } from '../db/schema/matches';
import type { Team } from '../db/schema/teams';

/**
 * Interface for generating tournament structures
 */
export interface TournamentGenerator {
	/**
	 * Get the scheduling mode this generator implements
	 */
	getMode(): SchedulingMode;

	/**
	 * Generate the bracket structure needed for this tournament type
	 * Returns an array of bracket IDs that were created
	 */
	generateBrackets(): Promise<string[]>;

	/**
	 * Generate all rounds for a specific bracket
	 * @param bracketId The bracket to generate rounds for
	 */
	generateRounds(bracketId: string): Promise<RoundInsert[]>;

	/**
	 * Generate all matches for a specific round
	 * @param roundId The round to generate matches for
	 * @param teams Array of team IDs participating in the tournament
	 */
	generateMatches(roundId: string, teams: Team[]): Promise<Match[]>;

	/**
	 * Generate the next round based on previous round results
	 * @param previousRoundId ID of the previous round
	 * @param bracketId ID of the bracket
	 */
	generateNextRound(previousRoundId: string, bracketId: string): Promise<RoundInsert | null>;
}

/**
 * Represents a team seeded in the tournament
 */
export interface SeededTeam {
	teamId: string;
	seed: number;
}

/**
 * Configuration for bracket generation
 */
export interface BracketConfig {
	name: string;
	bracketType: BracketType;
	position: number;
	isActive: boolean;
}

/**
 * Base class for all tournament generators
 */
export abstract class BaseTournamentGenerator implements TournamentGenerator {
	protected competitionId: string;
	protected teamsCount: number;

	constructor(competitionId: string, teamsCount: number) {
		this.competitionId = competitionId;
		this.teamsCount = teamsCount;
	}

	abstract getMode(): SchedulingMode;
	abstract generateBrackets(): Promise<string[]>;
	abstract generateRounds(bracketId: string): Promise<RoundInsert[]>;
	abstract generateMatches(roundId: string, teams: Team[]): Promise<Match[]>;
	abstract generateNextRound(
		previousRoundId: string,
		bracketId: string
	): Promise<RoundInsert | null>;

	/**
	 * Calculate the number of rounds needed for a single elimination tournament
	 */
	protected calculateRoundsNeeded(): number {
		return Math.ceil(Math.log2(this.teamsCount));
	}

	/**
	 * Calculate the number of matches in the first round
	 */
	protected calculateFirstRoundMatches(): number {
		const roundsNeeded = this.calculateRoundsNeeded();
		const fullBracketTeams = Math.pow(2, roundsNeeded);
		const byes = fullBracketTeams - this.teamsCount;
		return this.teamsCount - byes;
	}
}
