import type { RequestHandler } from './$types';
import type { RoundInsert } from '@/lib/server/db/schema/rounds';
import { SchedulingMode } from '@/types';
import { createTournamentGenerator } from '@/lib/server/tournament';
import { getBracketsByListIds } from '@/lib/server/db/queries/brackets';
import { getCompetitionWithAll } from '@/lib/server/db/queries/competitions';

export const POST: RequestHandler = async ({ request, params, locals }) => {
	// get body from request
	// get competition id from url
	const body = await request.json();
	let competitionId;
	try {
		competitionId = params.id;
	} catch {
		throw new Error('Invalid competition id');
	}
	if (competitionId === '') {
		throw new Error('Invalid competition id');
	}

	// get competition
	const competition = await getCompetitionWithAll(competitionId);

	if (!competition) {
		throw new Error('Competition not found');
	}

	if (competition.owner != locals.user.id) {
		throw new Error('YYou are not the owner of this competition');
	}

	let schedulingMode: SchedulingMode;
	let name: string;
	let size: number;

	try {
		schedulingMode = SchedulingMode[body.scheduling_mode as keyof typeof SchedulingMode];
		name = body.name;
		size = body.size;
	} catch {
		throw new Error('Invalid scheduling mode');
	}

	// generate bracket
	const tournamentGenerator = createTournamentGenerator(schedulingMode, competitionId, size);
	const bracketsIds = await tournamentGenerator.generateBrackets();
	if (bracketsIds.length === 0) {
		throw new Error('No brackets created');
	}
	const brackets = await getBracketsByListIds(bracketsIds, {});

	if (brackets.length === 0) {
		throw new Error('No brackets created');
	}

	let rounds: RoundInsert[] = [];

	const firstRoundOfBracket: RoundInsert[] = [];

	for (const bracketId of bracketsIds) {
		const roundInserts = await tournamentGenerator.generateRounds(bracketId);
		rounds = [...rounds, ...roundInserts];
		firstRoundOfBracket.push(roundInserts[0]);
	}

	// only generate first round matches of each bracket
	brackets.forEach(async (bracket, index) => {
		if (!firstRoundOfBracket[index].id) {
			throw new Error('First round of bracket not found');
		}
		await tournamentGenerator.generateMatches(firstRoundOfBracket[index].id, competition.teams);
	});

	return new Response(JSON.stringify(brackets), {
		headers: {
			'Content-Type': 'application/json'
		}
	});
};
